#
# spec file for package nodejs4
#
# Copyright (c) 2017 SUSE LINUX GmbH, Nuernberg, Germany.
#
# All modifications and additions to the file contributed by third parties
# remain the property of their copyright owners, unless otherwise agreed
# upon. The license for this file, and modifications and additions to the
# file, is the same license as for the pristine package itself (unless the
# license for the pristine package is not an Open Source License, in which
# case the license is the MIT License). An "Open Source License" is a
# license that conforms to the Open Source Definition (Version 1.9)
# published by the Open Source Initiative.

# Please submit bugfixes or comments via http://bugs.opensuse.org/
#

###########################################################
#
#   WARNING! WARNING! WARNING! WARNING! WARNING! WARNING!
#
# This spec file is generated from a template hosted at
# https://github.com/AdamMajer/nodejs-packaging
#
###########################################################

Name:           nodejs4
Version:        4.7.3
Release:        6.1

%if 0%{?sles_version} == 11
%define _libexecdir %{_exec_prefix}/lib
%endif

# Only Leap 42.2+, SLE 12 SP2+ and Tumbleweed have OpenSSL 1.0.2.
%if 0%{?suse_version} > 1320 || 0%{?sle_version} >= 120200
%bcond_with    intree_openssl
%else
%bcond_without intree_openssl
%endif

%if 0
%bcond_with    intree_cares
%else
%bcond_without intree_cares
%endif

%if 0%{?suse_version} >= 1200
%bcond_with    intree_icu
%else
%bcond_without intree_icu
%endif

%ifarch aarch64 ppc ppc64 ppc64le s390 s390x
%bcond_with    gdb
%else
%bcond_without gdb
%endif

# No binutils_gold on SLE12 (aarch64).
%ifarch aarch64

%if 0%{?sle_version} >= 120100 || 0%{?is_opensuse}
%bcond_without binutils_gold
%else
%bcond_with    binutils_gold
%endif

%else # aarch64

# No binutils_gold on s390x.
%ifarch s390x s390
%bcond_with    binutils_gold
%else
%bcond_without binutils_gold
%endif # s390x s390

%endif # aarch64

Summary:        Evented I/O for V8 JavaScript
License:        MIT
Group:          Development/Languages/NodeJS
Url:            https://nodejs.org
Source:         https://nodejs.org/dist/v%{version}/node-v%{version}.tar.xz
Source1:        https://nodejs.org/dist/v%{version}/SHASUMS256.txt
Source2:        https://nodejs.org/dist/v%{version}/SHASUMS256.txt.sig
Source3:        nodejs.keyring

## UPSTREAM PATCHES HERE, if any
Patch1:         8334.diff

## Our patches
# PATCH-FIX-OPENSUSE -- set correct path for dtrace if it is built
Patch101:       nodejs-libpath.patch
# PATCH-FIX-UPSTREAM -- use custom addon.gypi by default instead of
# downloading node source
Patch102:       node-gyp-addon-gypi.patch
# PATCH-FIX-SLE -- configure script uses Python check_output method
# which isn't included in Python 2.6 (used in SLE 11).
Patch103:       nodejs-sle11-python26-check_output.patch
# PATCH-FIX-OPENSUSE -- install user global npm packages to /usr/local
# instead of /usr
Patch104:       npm_search_paths.patch

%if %{with binutils_gold}
BuildRequires:  binutils-gold
%endif

%if 0%{?sles_version} == 11
BuildRequires:  gcc48-c++
%else
BuildRequires:  gcc-c++
%endif

BuildRequires:  curl
BuildRequires:  fdupes
BuildRequires:  pkg-config
BuildRequires:  procps
BuildRequires:  python
BuildRequires:  xz
BuildRequires:  zlib-devel

%if ! %{with intree_openssl}
BuildRequires:  openssl-devel >= 1.0.2
%endif

%if ! %{with intree_cares}
BuildRequires:  pkgconfig(libcares) >= 1.10.0
%endif

%if ! %{with intree_icu}
BuildRequires:  pkgconfig(icu-i18n) >= 52
%endif

Recommends:     npm(npm) = 2.15.11

#we need ABI virtual provides where SONAMEs aren't enough/not present so deps
#break when binary compatibility is broken
%global nodejs_abi 4.0
Provides:       nodejs(abi) = %{nodejs_abi}

#this corresponds to the "engine" requirement in package.json
Provides:       nodejs(engine) = %{version}

# Only one nodejs installed at a time
Provides:       nodejs = %{version}
Conflicts:      otherproviders(nodejs)

# For SLE11, to be able to use the certificate store we need to have properly
# symlinked certificates. The compatability symlinks are provided by the
# openssl1 library in the Security Module
%if 0%{?sles_version} == 11
Requires:       openssl1
%endif

# Building Node.js only makes sense on V8 architectures.
ExclusiveArch:  %{ix86} x86_64 armv7hl aarch64 ppc ppc64 ppc64le
BuildRoot:      %{_tmppath}/%{name}-%{version}-build

%description
Node.js is a JavaScript runtime built on Chrome's V8 JavaScript engine.
Node.js uses an event-driven, non-blocking I/O model that makes it
lightweight and efficient. Node.js' package ecosystem, npm, is the largest
ecosystem of open source libraries in the world.

%package devel
Summary:        Files needed for development of NodeJS platforms
Group:          Development/Languages/NodeJS
Provides:       nodejs-devel = %{version}
Requires:       %{name} = %{version}
Conflicts:      otherproviders(nodejs-devel)

%description devel
This package provides development headers for Node.js.

%package -n npm4
Summary:        Package manager for Node.js
Group:          Development/Languages/NodeJS
Requires:       %{name}-devel = %{version}
Provides:       nodejs-npm = %{version}
Obsoletes:      nodejs-npm < 5.3.1
Provides:       npm(npm) = 2.15.11
Provides:       npm = %{version}
Conflicts:      otherproviders(npm)

%description -n npm4
A package manager for Node.js that allows developers to install and
publish packages to a package registry.

%package docs
Summary:        Node.js API documentation
Group:          Documentation/Other
%if 0%{?suse_version} >= 1200
# using noarch subpackage seems to break debuginfo on older releases
BuildArch:      noarch
%endif

%description docs
The API documentation for the Node.js JavaScript runtime.

%prep
echo "`grep node-v%{version}.tar.xz %{S:1} | head -n1 | cut -c1-64`  %{S:0}" | sha256sum -c
%setup -q -n node-v%{version}
%if 4 != 7
%patch1 -p1
%endif
%patch101 -p1
%patch102 -p1
# Add check_output to configure script (not part of Python 2.6 in SLE11).
%if 0%{?sles_version} == 11
%patch103 -p1
%endif
%patch104 -p1
# Make sure nothing gets included from bundled deps:
# We only delete the source and header files, because
# the remaining build scripts are still used.
%if ! %{with intree_openssl}
find deps/openssl -name *.[ch] -delete
%endif

%if ! %{with intree_icu}
rm -rf deps/icu-small
%endif

%if ! %{with intree_cares}
find deps/cares -name *.[ch] -delete
%endif

find deps/zlib -name *.[ch] -delete

%build
# percent-configure pulls in something that confuses node's configure
# script, so we'll do it thus:
export CFLAGS="%{optflags}"
export CXXFLAGS="%{optflags}"
# It seems that GCC 4.8 is needed to compile on SLE 11 SP4.
%if 0%{?sles_version} == 11
export CXX="g++-4.8"
%endif

./configure \
    --prefix=%{_prefix} \
%if ! %{with intree_openssl}
    --shared-openssl \
%endif
    --shared-zlib \
%if ! %{with intree_cares}
    --shared-cares \
%endif
%if ! %{with intree_icu}
    --with-intl=system-icu \
%endif
%if %{with gdb}
    --gdb \
%endif
    --without-dtrace \
    --openssl-use-def-ca-store

make %{?_smp_mflags}

%install
make install DESTDIR=%{buildroot} %{?_smp_mflags}
rm %{buildroot}%{_datadir}/doc/node/gdbinit

# npm man page
rm -rf %{buildroot}%{_libdir}/node_modules/npm/man
mkdir -p %{buildroot}%{_mandir}/{man1,man5,man7}
install -m 644 deps/npm/man/man1/* %{buildroot}%{_mandir}/man1
install -m 644 deps/npm/man/man5/* %{buildroot}%{_mandir}/man5
install -m 644 deps/npm/man/man7/* %{buildroot}%{_mandir}/man7

#node-gyp needs common.gypi too
install -D -m 644 common.gypi \
       %{buildroot}%{_datadir}/node/common.gypi
# install addon-rpm.gypi
install -D -m 644 addon-rpm.gypi \
       %{buildroot}%{_libdir}/node_modules/npm/node_modules/node-gyp/addon-rpm.gypi

# Own nodejs_sitelib
#mkdir -p %{buildroot}%{_libdir}/node_modules/npm

# clean
# hidden files and directories
find %{buildroot}%{_libdir}/node_modules/npm -name ".*" -print0 | xargs -r0 -0 rm -rf --
# windows stuff
find %{buildroot}%{_libdir}/node_modules/npm -name "*.bat" -delete
find %{buildroot}%{_libdir}/node_modules/npm -name "*.cmd" -delete
# build stuff
find %{buildroot}%{_libdir}/node_modules/npm -name "Makefile" -delete
rm -rf %{buildroot}%{_libdir}/node_modules/npm/{test,scripts}
find %{buildroot}%{_libdir}/node_modules/npm/node_modules -name "*.sh" -delete
find %{buildroot}%{_libdir}/node_modules/npm -name "src" -print0 | xargs -0 rm -rf --
# remove examples/tests/benchmark stuff
find %{buildroot}%{_libdir}/node_modules/npm/node_modules -name "example*" -print0 | xargs -0 rm -rf --
find %{buildroot}%{_libdir}/node_modules/npm/node_modules -name "test*" -print0 | xargs -0 rm -rf --
find %{buildroot}%{_libdir}/node_modules/npm/node_modules -name "*_test.*" -delete
find %{buildroot}%{_libdir}/node_modules/npm/node_modules -type d -name "benchmark" -print0 | xargs -0 rm -rf --
# README.md LICENSE file permission
find %{buildroot}%{_libdir}/node_modules/npm -name "README.md" -exec chmod -x {} +
find %{buildroot}%{_libdir}/node_modules/npm -name "LICENSE" -exec chmod -x {} +
# browser.js is useless for npm cli
find %{buildroot}%{_libdir}/node_modules/npm -name "browser.js" -delete
# fix permissions
chmod 0644 %{buildroot}%{_libdir}/node_modules/npm/node_modules/request/node_modules/hawk/node_modules/sntp/lib/index.js \
   %{buildroot}%{_libdir}/node_modules/npm/node_modules/request/node_modules/qs/lib/parse.js \
   %{buildroot}%{_libdir}/node_modules/npm/node_modules/request/node_modules/hawk/lib/index.js \
   %{buildroot}%{_libdir}/node_modules/npm/node_modules/config-chain/index.js \
   %{buildroot}%{_libdir}/node_modules/npm/node_modules/request/node_modules/qs/lib/stringify.js \
   %{buildroot}%{_libdir}/node_modules/npm/node_modules/request/node_modules/hawk/lib/client.js \
   %{buildroot}%{_libdir}/node_modules/npm/node_modules/request/node_modules/hawk/lib/crypto.js \
   %{buildroot}%{_libdir}/node_modules/npm/node_modules/request/node_modules/hawk/lib/utils.js \
   %{buildroot}%{_libdir}/node_modules/npm/node_modules/request/index.js \
   %{buildroot}%{_libdir}/node_modules/npm/node_modules/request/node_modules/qs/lib/index.js \
   %{buildroot}%{_libdir}/node_modules/npm/node_modules/request/node_modules/hawk/node_modules/boom/lib/index.js \
   %{buildroot}%{_libdir}/node_modules/npm/node_modules/request/node_modules/hawk/lib/server.js \
   %{buildroot}%{_libdir}/node_modules/npm/node_modules/request/node_modules/hawk/node_modules/cryptiles/lib/index.js \
   %{buildroot}%{_libdir}/node_modules/npm/node_modules/request/node_modules/hawk/node_modules/sntp/index.js \
   %{buildroot}%{_libdir}/node_modules/npm/node_modules/request/node_modules/qs/lib/utils.js \
   %{buildroot}%{_libdir}/node_modules/npm/node_modules/uid-number/get-uid-gid.js \
   %{buildroot}%{_libdir}/node_modules/npm/bin/read-package-json.js

# file duplicates
%fdupes %{buildroot}%{_libdir}/node_modules/npm
%fdupes %{buildroot}%{_includedir}/node
%fdupes %{buildroot}%{_mandir}/man5

# Check that Node.js isn't completely broken.
%check
%{buildroot}%{_bindir}/node -e 'require("assert").strictEqual(process.versions.node, "%{version}");'

%files
%defattr(-, root, root)
%doc AUTHORS LICENSE *.md
%doc deps/v8/tools/gdbinit
%dir %{_libdir}/node_modules
%{_bindir}/node
%{_libdir}/node_modules
%{_mandir}/man1/node.1%{ext_man}
%exclude %{_libdir}/node_modules/npm

%files -n npm4
%defattr(-, root, root)
%{_bindir}/npm
%{_libdir}/node_modules/npm
%{_mandir}/man*/*npm*%{ext_man}
%{_mandir}/man5/package.json.5%{ext_man}
%{_mandir}/man7/semver.7%{ext_man}

%files devel
%defattr(-, root, root)
%{_includedir}/node
%{_datadir}/node
%dir %{_datadir}/systemtap
%dir %{_datadir}/systemtap/tapset
%{_datadir}/systemtap/tapset/node.stp

%files docs
%defattr(-,root,root)
%doc doc/api

%changelog
* Fri Feb  3 2017 adam.majer@suse.de
- New upstream LTS release 4.7.3
  * deps: upgrade openssl sources to 1.0.2k
    (CVE-2017-3731, CVE-2017-3732, CVE-2016-7055,
    bnc#1022085, bnc#1022086, bnc#1009528)
- No changes in LTS version 4.7.2
- Adjusted 8334.diff to be inline with accepted changes
- Merge nodejs4.changes from SLE and devel project
* Fri Jan  6 2017 qantas94heavy@gmail.com
- Add basic check that Node.js loads successfully to spec file
* Wed Jan  4 2017 qantas94heavy@gmail.com
- New upstream LTS release 4.7.1
  * build: shared library support is now working for AIX builds
  * repl: passing options to the repl will no longer overwrite
    defaults
  * timers: recanceling a cancelled timers will no longer throw
* Fri Dec  9 2016 qantas94heavy@gmail.com
- New upstream LTS version 4.7.0
  * build: introduce the configure --shared option for embedders
  * debugger: make listen address configurable in debugger server
  * dgram: generalized send queue to handle close, fixing a
    potential throw when dgram socket is closed in the
    listening event handler
  * http: introduce the 451 status code "Unavailable For
    Legal Reasons"
  * gtest: the test reporter now outputs tap comments as yamlish
  * tls: introduce secureContext for tls.connect (useful for
    caching client certificates, key, and CA certificates)
  * tls: fix memory leak when writing data to TLSWrap instance
    during handshake
  * src: node no longer aborts when c-ares initialization fails
- Modify 8334.diff:
  * ported and updated system CA store for the new node crypto code
- Refresh nodejs-libpath.patch
* Thu Dec  1 2016 qantas94heavy@gmail.com
- New upstream LTS version 4.6.2
  * build:
    + It is now possible to build the documentation from the release tarball.
  * buffer:
    + Buffer.alloc() will no longer incorrectly return a zero filled buffer
    when an encoding is passed.
  * deps:
    + Upgrade npm in LTS to 2.15.11.
  * repl:
    + Enable tab completion for global properties.
  * url:
    + url.format() will now encode all "#" in search.
* Wed Nov 23 2016 adam.majer@suse.de
- Add missing conflicts to base package. It's not possible to have
  concurrent nodejs installations.
* Fri Nov 18 2016 adam.majer@suse.de
- Package unification across various branches of NodeJS. Package
  for 4.x, 6.x and current (7.x) branches of NodeJS are now
  handled via GitHub repository.
- remove support-arm64-build.patch: no longer required
- remove nodejs-libpath64.patch: obsolete
* Tue Nov  8 2016 adam.majer@suse.de
- npm4 should provide versioned nodejs-npm and npm allowing
  nodejs-packaging to continue to function properly in Leap 42.2
  (bnc #1009011)
* Wed Oct 19 2016 qantas94heavy@gmail.com
- New upstream LTS version 4.6.1
  * c-ares: fix for single-byte buffer overwrite, CVE-2016-5180
    more information at https://c-ares.haxx.se/adv_20160929.html
    (bnc #1007728)
* Tue Oct  4 2016 adam.majer@suse.de
- npm4 now provides nodejs-npm to ease upgrades for Leap
* Thu Sep 29 2016 adam.majer@suse.de
- enable usage of system certificate store on SLE11SP4 by
  requiring openssl1 (boo#1000036)
- nodejs-libpath.patch:
  * adapt patch from main nodejs project so it builds on SLE11
- New upstream LTS version 4.6.0
  * openssl update (not applicable for SLE12SP2, Leap 42.2 and later)
    + upgrade to 1.0.2j (CVE-2016-6304, CVE-2016-2183, CVE-2016-2178,
    CVE-2016-6306, CVE-2016-7052)
    + remove support for dynamic 3rd party engine modules
  * http: Properly validate for allowable characters in input
    user data. This introduces a new case where throw may occur
    when configuring HTTP responses, users should already
    be adopting try/catch here. (CVE-2016-5325, bnc#985201)
  * tls: properly validate wildcard certificates
    (CVE-2016-7099, bnc#1001652)
  * buffer: Zero-fill excess bytes in new Buffer objects created
    with Buffer.concat()
* Fri Aug 26 2016 adam.majer@suse.de
- New upstream LTS version 4.5.0 (bnc#997405)
  * buffer:
    + backport new buffer constructor APIs to v4.x
    + backport --zero-fill-buffers cli option
    + ignore negative allocation lengths
  * build
    + add Intel Vtune profiling support
  * repl
    + copying tabs shouldn't trigger completion
  * src
    + add node::FreeEnvironment public API
  * test
    + run v8 tests from node tree
  * V8
    + Add post mortem data to improve object inspection and
    function's context variables inspection
  * upgrade libuv to 1.9.1
  * upgrade npm to 2.15.9
- 8334.diff
  * use system CA store instead of one provided by Node
- Refresh patches
* Wed Aug 10 2016 adam.majer@suse.de
- use system OpenSSL with Leap 42.2 and SLE12:SP2
- simplify source code integrity check
  + use GPG service instead of explicit BR
  + add empty checksum so GPG service is run - it's not detached signature
    like it thinks it is.
* Mon Jul  4 2016 adam.majer@suse.de
- rename patches to have a .patch suffix, for consistancy
- npm_search_paths.patch:
  Change defaultPrefix to /usr/local if it is detected to be
  /usr. This is in attempt to prevent globally installed npm-managed
  packages from installing into the zypper managed prefix.
- refreshed patches support-arm64-build.patch
- use upstream .xz instead of .gz tarball
* Fri Jul  1 2016 adam.majer@suse.de
- New upstream version 4.4.7
  * debugger:
    + All properties of an array (aside from length) can now be printed
    in the repl
  * Upgrade npm to 2.15.8 (Rebecca Turner)
  * Fix for a bug that became more prevalent with the stream changes
    that landed in v4.4.5. (Anna Henningsen). 'reset awaitDrain after manual
    .resume()'
  * V8:
    + Fix for a bug in crankshaft that was causing crashes on arm64
    (Myles Borins)
    +  Add missing classes to postmortem info such as JSMap and JSSet
    (evan.lucas)
- Add upstream release keyring
- Verify upstream sources during %%prep
* Mon Jun 27 2016 adam.majer@suse.de
- Use build flags to enable/disable gdb usage instead of
  configure script. Easier to find and change in future.
- Fix paths, and have to fix lots of paths because they
  are all more or less hardcoded relative paths.
- Renumber patches allowing upstream patches to be inserted
  before our own.
* Fri Jun 24 2016 adam.majer@suse.de
- New upstream version 4.4.6
  + fix buffer overflow vulnerability discovered in v8
  (CVE-2016-1669)
* Thu Jun 16 2016 adam.majer@suse.de
- Change detection of library paths from runtime to compile time.
  nodejs-libpath.patch, nodejs-libpath64.patch
* Wed Jun 15 2016 adam.majer@suse.de
- This package is in response to FATE#320396 and ECO#317945
  and references bnc#958943
  It's to be part of Web and Scripting Module
- Use build conditional for intree_openssl
- Fix permissions of some supplies javascript files - they are
  not executables
- General cleanup of the package
* Wed Jun 15 2016 adam.majer@suse.de
- Tighten dependencies so we don't end up with mixed versions
  installed.
* Tue Jun 14 2016 adam.majer@suse.de
- Dedup manpages
- Conflict with other providers of NodeJS packages. This is
  important if we want to provide NodeJS v6.x branch along with
  v4.x branch
* Mon Jun  6 2016 adam.majer@suse.de
- 'New' package of 4.x LTS branch of NodeJS, based on v6.2.1
  from Tumbleweed
-  Fix search paths to actually look where modules are installed
